import os
import re
import yaml
import json
import traceback

# Every file matching the pattern gets thrown into a Page object
class Page(object):
    def __init__(self, data, config):
        self.data = data
        self.config = config
        self.output_path = False
        self.fileExtension = self.config["data_format"]

    def get_page_content(self):
        return self.data["data"]

    def get_metadata(self):
        return self.data["meta"]

    def set_output_path(self, output_dir, output_file_extension):
        name = self.data["meta"]["relative_path"].replace(".%s" % (self.fileExtension), ".%s" % (output_file_extension), 1)
        path = "%s/%s" % (output_dir, name)
        self.output_path = re.sub("/{2,}", "/", path)
        self.data["meta"]["slug"] = self.output_path

    def get_output_path(self):
        return self.output_path

    def get_modified_time(self):
        return self.data["meta"]["mtime"]

    def get_file_path(self):
        return self.data["meta"]["file_path"]

    def get_mtime(self):
        if "mtime" in self.data["meta"]:
            return self.data["meta"]["mtime"]
        return 0

    def get_template(self):
        if "template" in self.data["meta"]:
            return self.data["meta"]['template']

        return self.config["default_template"]

    def before_render(self):
        return False;

    def on_save(self):
        return False;


import os
import sys
import argparse
import json
import importlib
import JinjaStaticRenderer as renderer
import DirectoryWalker as walker


defaults = {
    "generator_config": {
        "build_cache_file": "var/build.cache",
        "default_template": "base.j2",
        "templates_dir": "templates",
        "compiled_templates_file": "var/jinja",
        "compiled_templates_log": "var/jinja.log",
        "output_file_extension": "html",
        "output_dir": "www",
        "input_dir": "site",
        "data_format": "json",
        "ignore": [".", "_"],
        "site_vars":"env",
        "verbose": "false",
        "force": "false",
        "nocache": "false",
        "item_class": "Page"
    },
    "env": {
        "paths": {
            "css": "assets/css",
            "js": "assets/js",
            "assets": "assets",
            "images": "assets/images",
            "webroot": "www",
            "app": "app",
            "content": "content"
        }
    }
}


# Default config just loads the YAML config file
config = {
    'config_path':'wrangler.json',
}


# Generic little app bootstrapper thingy
class App():
    def __init__(self, config):
        self.config = config

    def parse_args(self, args=None):
        d = '\033[32mProcess recursive directories of JSON files through Jinja2 templates. \033[0m'
        parser = argparse.ArgumentParser(description=d)
        parser.add_argument('input_dir',
                            help='\033[34mInput directory such as `site/content`\033[0m')
        parser.add_argument('output_dir',
                            help='\033[34mOutput directory such as `www`\033[0m')
        parser.add_argument("-t", "--templates_dir",
                            help='\033[34mTemplate directory such as `templates`\033[0m')
        parser.add_argument("-c", "--config",
                            help='\033[34mPath to `config.json`\033[0m')
        parser.add_argument("-o", "--output_file_extension",
                            help='\033[34mType of files to output such as `html`\033[0m')
        parser.add_argument("-d", "--data_format",
                            help='\033[34mType of files to match in input_dir, such as `json`\033[0m')
        parser.add_argument("-v", "--verbose", action="store_true",
                            help='\033[34mPrint all the plumbing\033[0m')
        parser.add_argument("-f", "--force", action="store_true",
                            help='\033[34mSmash out all the templates regardless of mtime\033[0m')
        parser.add_argument("-n", "--nocache", action="store_true",
                            help='\033[34mTurn off data persistence\033[0m')
        return parser.parse_args(args)


    def main(self, args=None):
        args = self.parse_args(args)

        # If a config file is specified, load all config from there instead
        if (args.__dict__["config"] != None):
            config['config_path'] = args.__dict__["config"]

        if os.path.exists( config['config_path'] ):
            userConfig = json.load( file(config['config_path']) )
            config.update( userConfig["generator_config"] )
        else: 
            userConfig = defaults

        # Apply settings from the config file (if present)
        config.update( userConfig["generator_config"] )

        # Apply command line flags if set, ignore Nones.
        config.update((k, v) for k, v in args.__dict__.iteritems() if v is not None)
        
        # The site vars object is mapped to an item in the json object
        config["site_vars"] = userConfig[userConfig["generator_config"]["site_vars"]]

        self.nodes = walker.DirectoryWalker(self.config)
        self.renderer = renderer.JinjaStaticRenderer(self.config)

        if "views" in config:
            self.load_classes(config["views"])

        self.renderer.render(self.nodes.fetch())


    def load_classes(self, views): 

        classfiles = [os.path.join(dirpath, f)
            for dirpath, dirnames, files in os.walk(views)
            for f in files if f.endswith('.py')]

        path = list(sys.path)
        sys.path.insert(0, views)

        moduleNames = []

        for f in classfiles:
            moduleNames.append(os.path.basename(f.replace(".py", "")))

        if len(moduleNames) > 0:
            try:
                map(__import__, moduleNames);
            finally:
                # restore the syspath
                sys.path[:] = path 
    

wrangler = App(config)



def start():
    sys.exit(wrangler.main())

if __name__ == '__main__':
    start()



import os
import re
import yaml
import json
import traceback

class YamlParser():
    def __init__(self, fileName, input_dir, root):
        self.file_name, self.fileExtension = os.path.splitext(fileName)
        self.file_path = os.path.join(root, fileName)
        self.relative_path = self.file_path.replace(input_dir+"/", "")        
        self.output_filename = ""
        self.output_path = ""
        self.mtime = self.get_modified_time()
        self.file_contents = self.read_file(self.file_path)
        self.data = self.get_data(self.file_contents)
        self.item_class = self.get_itemclass()
        
    def read_file(self, file_path):
        source_file = open(file_path, 'r')
        file_contents = ""
        try:
            file_contents = source_file.read().decode('utf8')
        except:
            print "\033[31mTrouble reading %s\033[0m" % (source_file)
            traceback.print_exc() 
        
        return file_contents

    def get_data(self, file_contents):
        data = {
            "meta":{
                "title": None,
                "template": None,
                "description": None
            }
        }
        data.update(self.get_as_yaml(file_contents))

        data["meta"]["file_name"] = self.file_name
        data["meta"]["file_path"] = self.file_path
        data["meta"]["relative_path"] = self.file_path
        data["meta"]["mtime"] = self.mtime

        return data

    def get_itemclass(self):
        if self.data["meta"]:
            if "view" in self.data["meta"]:
                return self.data["meta"]["view"]

        return False

    def get_as_yaml(self, file_contents):
        yaml_data = {}
        try:
            yaml_data = yaml.load(file_contents)
        except:
            print "\033[31mCouldn't decode %s as YAML\033[0m" % (self.file_path)
            traceback.print_exc() 
    
        return yaml_data


    def get_modified_time(self):
        mtime = os.path.getmtime(self.file_path)
        return mtime

    def set_output_path(self, output_dir, output_file_extension):
        name = self.relative_path.replace(self.fileExtension, ".%s" % (output_file_extension), 1)
        path = "%s/%s" % (output_dir, name)
        self.output_path = re.sub("/{2,}", "/", path)
        self.output_filename = name
        self.meta["slug"] = self.output_filename

    def get_output_path(self):
        return self.output_path

    def get_file_path(self):
        return self.file_path

    def get_relative_path(self):
        return self.relative_path



import os
import subprocess
import shelve 
import sys 
import Page
from YamlParser import YamlParser

# Takes a directory and transforms the matching elements into the matching class.
class DirectoryWalker():
    def __init__(self, config):
        self.input_dir = config['input_dir']
        self.ignore_files = config['ignore']
        self.data_format = config['data_format']
        self.nocache = config['nocache']
        self.config = config


    def load_class(self, file_class):
        filename = "%s/%s.py" % (self.config["views"], file_class)

        if not os.path.exists(filename):
            raise Exception("path %s doesn't exist in %s" % (file_class, self.config["views"]))

        directory, module_name = os.path.split(filename)
        module_name = os.path.splitext(module_name)[0]

        path = list(sys.path)
        sys.path.insert(0, directory)

        try:
            module = __import__(module_name)
        finally:
            sys.path[:] = path # restore

        for name in dir(module):
            if module_name.split(".")[0] == name:
                return getattr(module, name)

        return Page

    def fetch(self):
        items = []

        # Not supported in python 2.6 hmmm
        # if sys.version_info > (2,7,0):
            # files = subprocess.check_output(["find", self.input_dir, "-name", "*.%s" % (self.data_format)], stderr=subprocess.STDOUT)
        # else:
        p = subprocess.Popen(["find", self.input_dir, "-name", "*.%s" % (self.data_format)], stdout=subprocess.PIPE)

        out, err = p.communicate()
        files = out.split("\n")

        if (self.config["nocache"]):
            try:
                os.remove("wrangler_cache.db")
            except OSError:
                pass

        shelf = shelve.open("wrangler_cache") 

        for f in files: 
            if f != '':

                file_mtime = os.path.getmtime(f);
                lib_needs_refresh = False

                if "views" in self.config and f in shelf and hasattr(shelf[f], "item_class"):
                    lib_path = self.config["views"];
                    lib_mtime = os.path.getmtime("%s/%s" % (lib_path, shelf[f].item_class))
                    
                    if lib_mtime < file_mtime:
                        lib_needs_refresh = True

                # If the key doesn't exist on the shelf, or the mtime is different, re-cache the object
                if (not shelf.has_key(f)) or (shelf[f].get_mtime() < file_mtime) or (self.nocache) or (lib_needs_refresh):

                    # Check if custom class is set, otherwise make it a page... 
                    page = YamlParser(f, self.input_dir, "");

                    if (page.item_class and "views" in self.config):
                        pageClass = self.load_class(page.item_class)
                    else:
                        pageClass = Page.Page

                    item = pageClass(page.data, self.config)

                    # Register the output path
                    item.set_output_path(self.config['output_dir'], self.config['output_file_extension'])
                    shelf[f] = item

                    print "\033[1;35mCaching \033[0m\033[2m%s\033[0m" % (f)

                items.append(shelf[f])
        try:
            shelf.sync()
        finally:
            shelf.close()

        return items


import os
import time
import markdown as md
import sys
import traceback
import glob

from docutils.core import publish_parts
from jinja2 import Environment, FileSystemLoader, meta, BaseLoader, TemplateNotFound, ChoiceLoader, Template, Markup
from SilentUndefined import SilentUndefined
from MarkdownExtension import MarkdownExtension


def rst_filter(s):
    return Markup(publish_parts(source=s, writer_name='html')['body'])

def markdown_filter(value):
    """
    Jinja2 markdown filter with support for extensions.
    """
    output = value
    marked = md.Markdown(
        extensions=['extra','meta','sane_lists'],
        output_format='html5',
        safe_mode=False 
        )
    out = marked.convert(output)
    return out

def filter_template_dir(filename):
    if filename.startswith("."):
        return False;
    return True;


class JinjaStaticRenderer():

    def __init__(self, config):
        self.config = config
        # Setup the env
        self.env = Environment(
            undefined=SilentUndefined,
            loader=FileSystemLoader(config['templates_dir']),
            extensions=[
                MarkdownExtension
            ],
            trim_blocks = True,
            lstrip_blocks = True
            )
        
        self.env.filters["markdown"] = markdown_filter
        self.env.filters['rst'] = rst_filter        
        self.template_trees = {}
        self.template_modified_times = {}
        
        var_path=os.path.dirname(self.config['compiled_templates_file'])

        if self.config["verbose"] == True:
            print "Ensure directory exists: \033[34m%s\033[0m" % (var_path)
            print "Ensure directory exists: \033[34m%s\033[0m" % (config['output_dir'])

        self.ensure_dir(var_path)
        self.ensure_dir(config['output_dir'])

        self.env.compile_templates(
            self.config['compiled_templates_file'],
            ignore_errors=False,
            filter_func=filter_template_dir 
            )

        if self.config["verbose"] == True:
            print "Loading templates from: \033[34m%s\033[0m" % (config['templates_dir'])
            print "Compile templates to .zip: \033[32m%s\033[0m" % (self.config['compiled_templates_file'])


    def get_template_mtime(self, template):
        """
        Request the template's mtime. If we don't have it yet, cache it.
        """
        if not template in self.template_modified_times:
            self.template_modified_times[template] = os.path.getmtime("%s/%s" % (self.config['templates_dir'], template) )

        return self.template_modified_times[template]

    def get_referenced_templates(self, template):
        """
        Request the template's Abstract Syntax Tree so we can find other template references.
        Store the reference list in a dictionary
        """
        if not template in self.template_trees:

            try: 
                ast = self.env.parse(self.env.loader.get_source(self.env, template))
                self.template_trees[template] = list(meta.find_referenced_templates(ast))
                
            except:
                self.template_trees[template] = list()

            if self.config["verbose"] == True:
                print "\033[34m%s\033[0m references templates \033[32m%s\033[0m" % (template, self.template_trees[template])

        return self.template_trees[template]

    def ensure_dir(self, dir):
        try:
            os.stat(dir)
        except:
            os.makedirs(dir)


    def render(self, data_source):
        self.log_data = []

        for item in data_source:
            template = self.load_template(item.get_template())

            if (self.should_render_item(item, template)):
                item.before_render()

                if self.config["verbose"] == True:
                    print "\033[34m%s\033[37m last built at: %s\033[0m" % (item.get_file_path(), item.get_modified_time())
                self.save(self.render_template(template, item));

        self.set_last_build_time(time.time())
        self.update_log(self.log_data, self.get_last_build_time())


    def render_template(self, template, item):
        return (template.render(data=item.get_page_content(), meta=item.get_metadata()), item)


    def save(self, data):
        item=data[1]
        html=data[0]

        filename = item.get_output_path()
        file_object = open(filename, "w")
        new_directory = os.path.split(filename)[0]

        try: 
            self.ensure_dir(new_directory)
            file_object.write(html.encode('utf8'))
            self.print_stdout(item.get_file_path(), filename, item.get_template())
            item.on_save()
        except:
            print "\033[31mCouldn't write %s\033[0m" % (filename)
            traceback.print_exc()
            self.log_item_saved(item.get_file_path(), item.get_template(), 0)
            return False
        finally:
            self.log_item_saved(item.get_file_path(), item.get_template(), 1)
            return True


    def load_template(self, template_name):
        template_object = False
        try:
            template_object = self.env.get_template(template_name)
        except:
            print "\033[1;91mCouldn't parse `%s`. Check the template for errors \033[0;37m" % (template)
            
            if self.config["verbose"] == True:
                traceback.print_exc()
            print "\033[0m"
            return False
        finally:
            return template_object


    def should_render_item(self, item, template):
        force_render = 0
        last_build_time = self.get_last_build_time()
        output_path = item.get_output_path();

        if last_build_time == 0:
            return True

        config_modified_time = self.get_config_modified_time()
        
        if last_build_time == 0 or config_modified_time >= last_build_time:
            return True

        if self.config["force"] == True:
            return True

        if output_path and not os.path.exists(output_path):
            return True
        
        if item.get_modified_time() >= last_build_time:
            return True

        template_modified_time = 0


        if template:

            referenced_templates = self.get_referenced_templates(template)
            template_modified_time = self.get_template_mtime(item.get_template())

            if template_modified_time >= last_build_time:
                return True

            ref_template_mtime = 0
            for t in referenced_templates:
                time = self.get_template_mtime(t)
                if time >= last_build_time:
                    ref_template_mtime = time
                    return True

        return False


    def print_stdout(self, original_path, new_path, template):
        print "\033[1;32mBuilding \033[0m\033[2m%s\033[0m > \033[34m%s \033[2m[%s]\033[0m" % (original_path, new_path, template)


    def update_log(self, items, last_modified_time):
        file = open(self.config['compiled_templates_log'], 'w')
        print>>file, "this_render=%s, prev_render=%s" % (time.time(), last_modified_time)

        for item in items:
            print>>file, "%s, %s, %s" % (item[0], item[2], item[1])

        if self.config["verbose"] == True:
            print "Wrote log to: \033[32m%s\033[0m" % (self.config['compiled_templates_log'])
    
    def log_item_saved(self, path, template, result):
        if not self.log_data:
            self.log_data = []
        self.log_data.append([path, template, result])

    def get_last_build_time(self):
        """
        Get the last run time. Useful for checking what stuff's changed since we
        last ran the script.
        """
        last_time = 0
        if os.path.exists(self.config['build_cache_file']):
            with open(self.config['build_cache_file'], "r") as buildFile: 
                lines = map(float, buildFile.readline().split())
                if len(lines) > 0:
                    last_time = lines[0]
        return last_time

    def get_config_modified_time(self):
        """
        Force a re-render if the config file has changed (global paths may be invalid, etc..)
        """
        mtime = 0
        if os.path.exists(self.config['config_path']):
            mtime = os.path.getmtime(self.config['config_path'])
        return mtime

    def set_last_build_time(self, update_time):
        """
        Save the last successful run time in a var
        """
        with open(self.config['build_cache_file'], "w") as buildFile:
            buildFile.write("%s" % (update_time))

    def get_global_vars(self):
        return self.config["site_vars"]

    def go(self, data):
        """
        Request some data from an endpoint/directory
        """
        self.render(data)